SellOMatic = LibStub("AceAddon-3.0"):NewAddon("SellOMatic", "AceEvent-3.0", "AceConsole-3.0");
local SellOMatic = _G.SellOMatic
--local L = LibStub("AceLocale-3.0"):GetLocale("SellOMatic");

local function getOption(info)
	return (info.arg and SellOMatic.db.profile[info.arg] or SellOMatic.db.profile[info[#info]]);
end;

local function setOption(info, value)
	local key = info.arg or info[#info];
	SellOMatic.db.profile[key] = value;
end;

local options = {
	name = "Sell-O-Matic",
	handler = SellOMatic,
	type = "group",
	args = {
		general = {
			type = "group",
			name = "General Options",
			guiInline = true,
			args = {
				autoSell = {
					name = "Auto sell",
					desc = "Toggle ON/OFF automatic mode.",
					order = 1,
					type = "toggle",
					get = getOption,
					set = setOption,
				},
				showFullInfo = {
					name = "Information type",
					desc = "Choose the type of information",
					order = 2,
					type = "select",
					values = {
						["1-FULL"] = "FULL",
						["2-SUMMARY"] = "SUMMARY",
						["3-LITE"] = "LITE",
					},
					style = "dropdown",
					get = getOption,
					set = setOption,
				},
				showDetail = {
					name = "Show detail",
					desc = "Toggle ON/OFF item sell detail.",
					order = 3,
					type = "toggle",
					get = getOption,
					set = setOption,
				},
				showAdditionalButtons = {
					name = "Show +/- buttons",
					desc = "Toggle ON/OFF showing add/del buttons.",
					order = 10,
					type = "toggle",
					get = getOption,
					set = setOption,
				},
			},
		},
		scan = {
			name = "scan",
			hidden = true,
			type = "execute",
			func = "Scan",
		},
		show = {
			name = "show",
			hidden = true,
			type = "execute",
			func = "Show_Shopping_List",
		},
		add = {
			name = "add",
			hidden = true,
			type = "input",
			set = "AddItemTest",
			get = false,
		},
		showlist = {
			name = "showlist",
			hidden = true,
			type = "execute",
			func = "ShowSOMList",
		},
		clean = {
			name = "clean",
			hidden = true,
			type = "execute",
			func = "Clean_Shopping_List",
		},
		clear = {
			name = "clean",
			hidden = true,
			type = "execute",
			func = "Clear_Lists",
		},
		frame = {
			name = "frame",
			hidden = true,
			type = "execute",
			func = "AddItem",
		},
		dframe = {
			name = "dframe",
			hidden = true,
			type = "execute",
			func = "DelItem",
		},
	},
}

local defaults = {
	profile = {
		-- CORE
		autoSell = false,
		showFullInfo = "FULL",
		showDetail = false,
		showAdditionalButtons = false,
		-- ITEM
		allowWhite = false,
		allowGreen = false,
		allowBlue = false,
		allowEpic = false,
		allowBoP = false,
		allowBoE = false,
		iLevelValue = 1,
		useILevel = false,
		-- LIST
		sellList = {},
		saveList = {},
		caseSensitiveList = false,
		list_allowBoP = false,
		list_allowBoE = false,
		list_allowWhite = false,
		list_allowGreen = false,
		list_allowBlue = false,
		list_allowEpic = false,
		list_useILevel = false,
		list_iLevelValue = 1,
	},
}

local modules = { destroy = 1, list = 1, item = 1 };
-- item_container: Name, Link, Rarity, Level, Price
local shopping_list, SOMNone, duplicates;

function SellOMatic:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SellOMaticDB", defaults, "Default");
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SellOMatic", options);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SellOMatic", "Sell-O-Matic");
	local SOM_List = LoadAddOn("SellOMatic_List");
	if not SOM_List then modules.list = 0 end;
	local SOM_Item = LoadAddOn("SellOMatic_Item");
	if not SOM_Item then modules.item = 0 end;
	local SOM_Destroy = LoadAddOn("SellOMatic_Destroy");
	if not SOM_Destroy then modules.destroy = 0 end;
	self:RegisterChatCommand("sellomatic", "ChatCommand");
	self:RegisterChatCommand("som", "ChatCommand");
	SellOMatic:Create_SOMNone();
end;

function SellOMatic:OnEnable()
	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("MERCHANT_CLOSED");
	SellButton:Hide();
	shopping_list = {};
end;

function SellOMatic:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SellOMatic, "sellomatic", "SellOMatic", input);
	end;
end;

function SellOMatic:MERCHANT_SHOW()
	SellButton:Show();
	SellOMatic:Scan();
	if #shopping_list == 0 then
		SOMNone:Show();
	end;
end;

function SellOMatic:MERCHANT_CLOSED()
	SellButton:Hide();
	SOMNone:Hide();
	SellOMatic:Clean_Shopping_List();
end;

function SellOMatic:Sell()
	self:Print("Click!");
end;

function SellOMatic:Scan()
	SellOMatic:Scan_Junk();
	if modules.item then SellOMatic:Scan_Item() end;
	if modules.list then SellOMatic:Scan_List() end;
	SellOMatic:Check_Shopping_List();
end;

function SellOMatic:Scan_Junk()
	local bag, slot, name, itemName, itemLink, itemRarity, itemSellPrice, itemStackCount;
	for bag = 0,4,1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			name = GetContainerItemLink(bag,slot);
			if name ~= nil then
				itemName, itemLink, itemRarity, itemLevel, _, _, _, itemStackCount, _, _, itemSellPrice = GetItemInfo(name);
				if itemRarity == 0 then
					if itemStackCount > 1 then
						itemSellPrice = itemSellPrice * GetItemCount(itemName);
					end;
					SellOMatic:Add_Shopping_List(itemName,itemLink,itemRarity,itemLevel,itemSellPrice);
				end;
			end;
		end;
	end;
end;

function SellOMatic:Sell_Shopping_List()
	local i,bag,slot,name,itemLink;
	local num = 0;
	local profit = SellOMatic:ShowProfit();
	for bag = 0,4,1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			name = GetContainerItemLink(bag,slot);
			if name ~= nil then
				itemName, itemLink = GetItemInfo(name);
				for i = 1, #shopping_list, 1 do
					if shopping_list[i][1] == itemName then
						-- Sell item
						-- UseContainerItem(bag,slot);
						SellOMatic:Print("Selling: "..itemLink);
						num = num + 1;
					end;
				end;
			end;
		end;
	end;
	SellOMatic:Print(num.." ".."item(s) sold");
	SellOMatic:Print("You've earned:".." "..SellOMatic:FormatMoneyFull(profit));
end;

-- Scan items and call other modules
-- Save all items in array
-- Display merchant animation if array has items
-- Display merchant value if array has items
-- Sell items

function SellOMatic:Add_Shopping_List(name,link,rarity,level,price)
	shopping_list[#shopping_list+1] = { name,link,rarity,level,price };
end;

function SellOMatic:Del_Shopping_List(index)
	table.remove(shopping_list, index);
end;

function SellOMatic:Get_Shopping_List()
	return shopping_list;
end;

function SellOMatic:Check_Shopping_List()
--	local x,y;
--	duplicates = {};
--	if shopping_list[2] ~= nil then
--		for x=1, #shopping_list, 1 do
--			SellOMatic:CountItems(shopping_list[x][1]);
--		end;
--	end;
--	for x=1, #duplicates, 1 do
--		self:Print(duplicates[x]);
--		SellOMatic:Del_Shopping_List(duplicates[x]);
--	end;
	SellOMatic:RemoveClones();
end;

function SellOMatic:RemoveClones()
	local x,found;
	for x=1, #shopping_list-1, 1 do
		found = SellOMatic:CheckClones();
		if found ~= 0 then
			SellOMatic:Del_Shopping_List(found);
		end;
	end;
end;

function SellOMatic:CheckClones()
	local x,y;
	local found = 0;
	for x=1, #shopping_list-1, 1 do
		for y=x+1, #shopping_list, 1 do
			if shopping_list[x][1] == shopping_list[y][1] then
				found = y;
			end;
		end;
	end;
	return found;
end;

function SellOMatic:ShowProfit()
	local profit = 0;
	local x;
	for x=1, #shopping_list, 1 do
		profit = profit + shopping_list[x][5];
	end;
	return profit;
end;

function SellOMatic:Clean_Shopping_List()
	shopping_list = {};
end;

function SellOMatic:Clear_Lists()
	SellOMatic.db.profile.sellList = {};
	SellOMatic.db.profile.saveList = {};	
end;

function SellOMatic:Create_SOMNone()
	SOMNone = CreateFrame("Frame","SOMNone",MerchantFrame);
	SOMNone:ClearAllPoints();
	SOMNone:SetPoint("TOPRIGHT",MerchantFrame,"TOPRIGHT",-43,-41);
	SOMNone:SetHeight(30);
	SOMNone:SetWidth(30);
	SOMNone:SetFrameStrata("DIALOG");
	SOMNone.texture = SOMNone:CreateTexture()
	SOMNone.texture:SetAllPoints(SOMNone);
	SOMNone.texture:SetTexture(0.3,0.3,0.3,0.7);
	SOMNone:Hide();
end;

function SellOMatic:SOMButtonTooltip_Show()
	GameTooltip_SetDefaultAnchor(GameTooltip,MerchantFrame);
	GameTooltip:ClearAllPoints();
	GameTooltip:SetPoint("TOPLEFT",MerchantFrame,"TOPRIGHT",-30,-40);
	GameTooltip:SetText(GetCoinTextureString(SellOMatic:ShowProfit()),1,1,1);
	GameTooltip:Show();
end;

function SellOMatic:SOMButtonTooltip_Hide()
	GameTooltip:Hide();
end;

-- Debug functions
function SellOMatic:Show_Shopping_List()
	local i;
	for i=1,#shopping_list,1 do
		self:Print("["..i.."] "..shopping_list[i][2].." "..shopping_list[i][5]);
	end;
end;
